/*
* PM - 2018
* AirKeys
* Barbu Tudor Antonio
* 332CB
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define NOTES 8

int octave = 1;
int effect = 0;

double freq[NOTES] = {
	262, 294, 330, 349, 392, 440, 494, 524
};

ISR(PCINT1_vect)
{
	if ((PINB & (1 << PB5)) == 0) {
		if (octave > 0) {
			octave -= 1;
			for (int i = 0; i < NOTES; i++) {
				freq[i] /= 2;
			}
		}
	}
	else if ((PINB & (1 << PB6)) == 0) {
		if (octave < 2) {
			octave += 1;
			for (int i = 0; i < NOTES; i++) {
				freq[i] *= 2;
			}
		}
	}
	else if ((PINB & (1 << PB7)) == 0) {
		effect ^= 1;
	}
	_delay_ms(50);
}

void ADC_init(void)
{
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while (ADCSRA & (1 << ADSC));

	return ADC;
}

double ADC_voltage(int raw)
{
	return raw * 5 / 1024.;
}

void init(void)
{
	/* Speaker */
	DDRB |= (1 << PB2);
	PORTB |= (1 << PB2);

	/* Buttons */
	DDRB &= ~(1 << PB5);
	DDRB &= ~(1 << PB6);
	DDRB &= ~(1 << PB7);
	PORTB |= (1 << PB5);
	PORTB |= (1 << PB6);
	PORTB |= (1 << PB7);

	/* ADC Pins */
	DDRA = 0;
	PORTA = 0xff;

	/* Interrupt */
	PCICR = 0x02;
	PCMSK1 = 0xe0;
}

void buzzer(int note) {
	int i;
	int duration = 100;
	
	int period = 1000000 / freq[note];
	while (duration--) {
		PORTB &= ~(1 << PB2);
		for (i = 0; i < period / 2; ++i)
			_delay_us(1);

		PORTB |= (1 << PB2);
		for (i = 0; i < period / 2; ++i)
			_delay_us(1);
	}

	if (!effect)
		return;

	duration = 100;
	while (duration--) {
		PORTB &= ~(1 << PB2);
		for (i = 0; i < period; ++i)
			_delay_us(1);

		PORTB |= (1 << PB2);
		for (i = 0; i < period; ++i)
			_delay_us(1);
	}
	duration = 100;
	while (duration--) {
		PORTB &= ~(1 << PB2);
		for (i = 0; i < period / 2; ++i)
			_delay_us(1);

		PORTB |= (1 << PB2);
		for (i = 0; i < period / 2; ++i)
			_delay_us(1);
	}
}

void sing(void)
{
	while (1) {
		for (int i = 0; i < NOTES; ++i) {
			double value = ADC_voltage(ADC_get(i));

			if (value < 4.5)
				buzzer(i);
		}
	}
}

int main(void)
{
	init();
	ADC_init();
	sei();

	sing();

	return 0;
}